
#ifndef KBIBTEXNETWORKING_EXPORT_H
#define KBIBTEXNETWORKING_EXPORT_H

#ifdef KBIBTEXNETWORKING_STATIC_DEFINE
#  define KBIBTEXNETWORKING_EXPORT
#  define KBIBTEXNETWORKING_NO_EXPORT
#else
#  ifndef KBIBTEXNETWORKING_EXPORT
#    ifdef kbibtexnetworking_EXPORTS
        /* We are building this library */
#      define KBIBTEXNETWORKING_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KBIBTEXNETWORKING_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KBIBTEXNETWORKING_NO_EXPORT
#    define KBIBTEXNETWORKING_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KBIBTEXNETWORKING_DEPRECATED
#  define KBIBTEXNETWORKING_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KBIBTEXNETWORKING_DEPRECATED_EXPORT
#  define KBIBTEXNETWORKING_DEPRECATED_EXPORT KBIBTEXNETWORKING_EXPORT KBIBTEXNETWORKING_DEPRECATED
#endif

#ifndef KBIBTEXNETWORKING_DEPRECATED_NO_EXPORT
#  define KBIBTEXNETWORKING_DEPRECATED_NO_EXPORT KBIBTEXNETWORKING_NO_EXPORT KBIBTEXNETWORKING_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KBIBTEXNETWORKING_NO_DEPRECATED
#    define KBIBTEXNETWORKING_NO_DEPRECATED
#  endif
#endif

#endif /* KBIBTEXNETWORKING_EXPORT_H */
