
#ifndef KBIBTEXGUI_EXPORT_H
#define KBIBTEXGUI_EXPORT_H

#ifdef KBIBTEXGUI_STATIC_DEFINE
#  define KBIBTEXGUI_EXPORT
#  define KBIBTEXGUI_NO_EXPORT
#else
#  ifndef KBIBTEXGUI_EXPORT
#    ifdef kbibtexgui_EXPORTS
        /* We are building this library */
#      define KBIBTEXGUI_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KBIBTEXGUI_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KBIBTEXGUI_NO_EXPORT
#    define KBIBTEXGUI_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KBIBTEXGUI_DEPRECATED
#  define KBIBTEXGUI_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KBIBTEXGUI_DEPRECATED_EXPORT
#  define KBIBTEXGUI_DEPRECATED_EXPORT KBIBTEXGUI_EXPORT KBIBTEXGUI_DEPRECATED
#endif

#ifndef KBIBTEXGUI_DEPRECATED_NO_EXPORT
#  define KBIBTEXGUI_DEPRECATED_NO_EXPORT KBIBTEXGUI_NO_EXPORT KBIBTEXGUI_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KBIBTEXGUI_NO_DEPRECATED
#    define KBIBTEXGUI_NO_DEPRECATED
#  endif
#endif

#endif /* KBIBTEXGUI_EXPORT_H */
