<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KBibTeX" version="1">
<MenuBar>
  <Menu name="file"><text>File</text>

    <Action name="file_new" />
    <Action name="file_open" />
    <Action name="file_open_recent" />
    <Action name="file_close" />
    <DefineGroup name="save_merge" append="save_merge" />
    <Separator/>

    <Action name="file_quit"/>
  </Menu>
  <Menu name="settings">
    <Action name="settings_shown_panels" append="show_merge" />
  </Menu>
  <Merge />
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
    <Action name="file_new" />
    <Action name="file_open" />
    <Action name="file_close" />
    <Separator/>
    <Merge/>
</ToolBar>
</kpartgui>
