
#ifndef KBIBTEXPROCESSING_EXPORT_H
#define KBIBTEXPROCESSING_EXPORT_H

#ifdef KBIBTEXPROCESSING_STATIC_DEFINE
#  define KBIBTEXPROCESSING_EXPORT
#  define KBIBTEXPROCESSING_NO_EXPORT
#else
#  ifndef KBIBTEXPROCESSING_EXPORT
#    ifdef kbibtexprocessing_EXPORTS
        /* We are building this library */
#      define KBIBTEXPROCESSING_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KBIBTEXPROCESSING_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KBIBTEXPROCESSING_NO_EXPORT
#    define KBIBTEXPROCESSING_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KBIBTEXPROCESSING_DEPRECATED
#  define KBIBTEXPROCESSING_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KBIBTEXPROCESSING_DEPRECATED_EXPORT
#  define KBIBTEXPROCESSING_DEPRECATED_EXPORT KBIBTEXPROCESSING_EXPORT KBIBTEXPROCESSING_DEPRECATED
#endif

#ifndef KBIBTEXPROCESSING_DEPRECATED_NO_EXPORT
#  define KBIBTEXPROCESSING_DEPRECATED_NO_EXPORT KBIBTEXPROCESSING_NO_EXPORT KBIBTEXPROCESSING_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KBIBTEXPROCESSING_NO_DEPRECATED
#    define KBIBTEXPROCESSING_NO_DEPRECATED
#  endif
#endif

#endif /* KBIBTEXPROCESSING_EXPORT_H */
