// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KBIBTEX_VERSION_H
#define KBIBTEX_VERSION_H

#define KBIBTEX_VERSION_STRING "0.10.0"
#define KBIBTEX_VERSION_MAJOR 0
#define KBIBTEX_VERSION_MINOR 10
#define KBIBTEX_VERSION_PATCH 0
#define KBIBTEX_VERSION ((0<<16)|(10<<8)|(0))

#endif
