
#ifndef KBIBTEXCONFIG_EXPORT_H
#define KBIBTEXCONFIG_EXPORT_H

#ifdef KBIBTEXCONFIG_STATIC_DEFINE
#  define KBIBTEXCONFIG_EXPORT
#  define KBIBTEXCONFIG_NO_EXPORT
#else
#  ifndef KBIBTEXCONFIG_EXPORT
#    ifdef kbibtexconfig_EXPORTS
        /* We are building this library */
#      define KBIBTEXCONFIG_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KBIBTEXCONFIG_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KBIBTEXCONFIG_NO_EXPORT
#    define KBIBTEXCONFIG_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KBIBTEXCONFIG_DEPRECATED
#  define KBIBTEXCONFIG_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KBIBTEXCONFIG_DEPRECATED_EXPORT
#  define KBIBTEXCONFIG_DEPRECATED_EXPORT KBIBTEXCONFIG_EXPORT KBIBTEXCONFIG_DEPRECATED
#endif

#ifndef KBIBTEXCONFIG_DEPRECATED_NO_EXPORT
#  define KBIBTEXCONFIG_DEPRECATED_NO_EXPORT KBIBTEXCONFIG_NO_EXPORT KBIBTEXCONFIG_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KBIBTEXCONFIG_NO_DEPRECATED
#    define KBIBTEXCONFIG_NO_DEPRECATED
#  endif
#endif

#endif /* KBIBTEXCONFIG_EXPORT_H */
