
#ifndef KBIBTEXDATA_EXPORT_H
#define KBIBTEXDATA_EXPORT_H

#ifdef KBIBTEXDATA_STATIC_DEFINE
#  define KBIBTEXDATA_EXPORT
#  define KBIBTEXDATA_NO_EXPORT
#else
#  ifndef KBIBTEXDATA_EXPORT
#    ifdef kbibtexdata_EXPORTS
        /* We are building this library */
#      define KBIBTEXDATA_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KBIBTEXDATA_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KBIBTEXDATA_NO_EXPORT
#    define KBIBTEXDATA_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KBIBTEXDATA_DEPRECATED
#  define KBIBTEXDATA_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KBIBTEXDATA_DEPRECATED_EXPORT
#  define KBIBTEXDATA_DEPRECATED_EXPORT KBIBTEXDATA_EXPORT KBIBTEXDATA_DEPRECATED
#endif

#ifndef KBIBTEXDATA_DEPRECATED_NO_EXPORT
#  define KBIBTEXDATA_DEPRECATED_NO_EXPORT KBIBTEXDATA_NO_EXPORT KBIBTEXDATA_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KBIBTEXDATA_NO_DEPRECATED
#    define KBIBTEXDATA_NO_DEPRECATED
#  endif
#endif

#endif /* KBIBTEXDATA_EXPORT_H */
