<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kbibtexpart" version="1">
<MenuBar>
  <Menu name="file"><text>File</text>
    <Action name="file_save" group="save_merge" />
    <Action name="file_save_as" group="save_merge" />
    <Action name="file_save_copy_as" group="save_merge" />
  </Menu>
  <Menu name="edit"><text>Edit</text>
    <Action name="edit_cut" group="edit_paste_merge" />
    <Action name="edit_copy" group="edit_paste_merge" />
    <Action name="edit_copy_references" group="edit_paste_merge" />
    <Action name="edit_paste" group="edit_paste_merge" />
    <Action name="edit_delete" group="edit_paste_merge" />
  </Menu>
  <Menu name="element"><text>Element</text>
    <Action name="element_new" />
    <Separator/>
    <Action name="element_edit" />
    <Action name="element_viewdocument" />
    <Separator/>
    <Action name="element_findpdf" />
    <Action name="entry_applydefaultformatstring" />
    <Action name="entry_colorlabel" />
    <Separator/>
    <Action name="findduplicates" />
    <Separator/>
    <Action name="sendtolyx" />
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar"><text>Main Toolbar</text>
  <Action name="file_save" group="file_operations" />
  <Action name="file_save_as" group="file_operations" />
  <Separator/>
  <Action name="element_new" />
  <Separator/>
  <Action name="findduplicates" />
  <Separator/>
  <Action name="sendtolyx" />
</ToolBar>
</kpartgui>
