
#ifndef KBIBTEXGLOBAL_EXPORT_H
#define KBIBTEXGLOBAL_EXPORT_H

#ifdef KBIBTEXGLOBAL_STATIC_DEFINE
#  define KBIBTEXGLOBAL_EXPORT
#  define KBIBTEXGLOBAL_NO_EXPORT
#else
#  ifndef KBIBTEXGLOBAL_EXPORT
#    ifdef kbibtexglobal_EXPORTS
        /* We are building this library */
#      define KBIBTEXGLOBAL_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KBIBTEXGLOBAL_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KBIBTEXGLOBAL_NO_EXPORT
#    define KBIBTEXGLOBAL_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KBIBTEXGLOBAL_DEPRECATED
#  define KBIBTEXGLOBAL_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KBIBTEXGLOBAL_DEPRECATED_EXPORT
#  define KBIBTEXGLOBAL_DEPRECATED_EXPORT KBIBTEXGLOBAL_EXPORT KBIBTEXGLOBAL_DEPRECATED
#endif

#ifndef KBIBTEXGLOBAL_DEPRECATED_NO_EXPORT
#  define KBIBTEXGLOBAL_DEPRECATED_NO_EXPORT KBIBTEXGLOBAL_NO_EXPORT KBIBTEXGLOBAL_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KBIBTEXGLOBAL_NO_DEPRECATED
#    define KBIBTEXGLOBAL_NO_DEPRECATED
#  endif
#endif

#endif /* KBIBTEXGLOBAL_EXPORT_H */
