
#ifndef KBIBTEXIO_EXPORT_H
#define KBIBTEXIO_EXPORT_H

#ifdef KBIBTEXIO_STATIC_DEFINE
#  define KBIBTEXIO_EXPORT
#  define KBIBTEXIO_NO_EXPORT
#else
#  ifndef KBIBTEXIO_EXPORT
#    ifdef kbibtexio_EXPORTS
        /* We are building this library */
#      define KBIBTEXIO_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KBIBTEXIO_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KBIBTEXIO_NO_EXPORT
#    define KBIBTEXIO_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KBIBTEXIO_DEPRECATED
#  define KBIBTEXIO_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KBIBTEXIO_DEPRECATED_EXPORT
#  define KBIBTEXIO_DEPRECATED_EXPORT KBIBTEXIO_EXPORT KBIBTEXIO_DEPRECATED
#endif

#ifndef KBIBTEXIO_DEPRECATED_NO_EXPORT
#  define KBIBTEXIO_DEPRECATED_NO_EXPORT KBIBTEXIO_NO_EXPORT KBIBTEXIO_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KBIBTEXIO_NO_DEPRECATED
#    define KBIBTEXIO_NO_DEPRECATED
#  endif
#endif

#endif /* KBIBTEXIO_EXPORT_H */
